;;; guile-irrlicht --- FFI bindings for Irrlicht Engine
;;; Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>
;;;
;;; This file is part of guile-irrlicht.
;;;
;;; Guile-irrlicht is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; Guile-irrlicht is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with guile-irrlicht.  If not, see
;;; <http://www.gnu.org/licenses/>.


(define-module (irrlicht video)
  #:use-module (oop goops)
  #:use-module (ice-9 optargs)
  #:use-module (irrlicht base)
  #:use-module (irrlicht foreign))


;; ITexture
(define-class <texture> (<irrlicht-base>)
  (irr-class #:init-value "ITexture"))

(export <texture>)


;; IVideoDriver
(define-class <video-driver> (<irrlicht-base>)
  (irr-class #:init-value "IVideoDriver"))

(define-method (begin-scene (video-driver <video-driver>) . rest)
  (let-keywords rest #f
        ((back-buffer #t)
         (z-buffer #t)
         (color '(255 0 0 0))
         video-data
         source-rect)
    ((get-irrlicht-proc "beginScene" video-driver)
     video-driver
     back-buffer
     z-buffer
     color
     video-data
     source-rect)))

(define-method (end-scene (video-driver <video-driver>))
  ((get-irrlicht-proc "endScene" video-driver)
   video-driver))

(define-method (get-fps (video-driver <video-driver>))
  (let ((getFPS (get-irrlicht-proc "getFPS" video-driver)))
    (getFPS video-driver)))

(define-method (get-name (video-driver <video-driver>))
  (let ((getName (get-irrlicht-proc "getName" video-driver)))
    (getName video-driver)))

(define-method (get-texture (video-driver <video-driver>) filename)
  (make <texture>
    #:irr-pointer
    ((get-irrlicht-proc "getTexture" video-driver)
     video-driver
     filename)))

(export <video-driver> begin-scene end-scene get-fps get-name get-texture)


;; S3DVertex
(define-class <vertex3d> (<irrlicht-base>)
  (irr-class #:init-value "S3DVertex"))

(define (make-vertex3d position normal color tcoords)
  (let ((S3DVertex_make (get-irrlicht-proc "S3DVertex_make")))
    (make <vertex3d>
      #:irr-pointer
      (S3DVertex_make position normal color tcoords))))

(export <vertex3d> make-vertex3d)
